@@ -80,6 +80,10 @@ struct _TrackerSparqlConnectionClass
                                                       const gchar              *sparql,
                                                       GCancellable             *cancellable,
                                                       GError                  **error);
+        TrackerSparqlStatement * (* update_statement) (TrackerSparqlConnection  *connection,
+                                                       const gchar              *sparql,
+                                                       GCancellable             *cancellable,
+                                                       GError                  **error);
 	TrackerNotifier * (* create_notifier) (TrackerSparqlConnection *connection);
 
 	void (* close) (TrackerSparqlConnection *connection);
@@ -259,6 +263,17 @@ struct _TrackerSparqlStatementClass
         GInputStream * (* serialize_finish) (TrackerSparqlStatement  *stmt,
                                              GAsyncResult            *res,
                                              GError                 **error);
+
+        gboolean (* update) (TrackerSparqlStatement  *stmt,
+                             GCancellable            *cancellable,
+                             GError                 **error);
+        void (* update_async) (TrackerSparqlStatement *stmt,
+                               GCancellable           *cancellable,
+                               GAsyncReadyCallback     callback,
+                               gpointer                user_data);
+        gboolean (* update_finish) (TrackerSparqlStatement  *stmt,
+                                    GAsyncResult            *res,
+                                    GError                 **error);
 };
 
 struct _TrackerNotifierClass {
@@ -276,6 +291,16 @@ struct _TrackerBatchClass {
 	void (* add_resource) (TrackerBatch    *batch,
 			       const gchar     *graph,
 			       TrackerResource *resource);
+	void (* add_statement) (TrackerBatch           *batch,
+	                        TrackerSparqlStatement *stmt,
+	                        guint                   n_values,
+	                        const gchar            *variable_names[],
+	                        const GValue            values[]);
+	void (* add_rdf) (TrackerBatch            *batch,
+	                  TrackerDeserializeFlags  flags,
+	                  TrackerRdfFormat         format,
+	                  const gchar             *default_graph,
+	                  GInputStream            *stream);
 	gboolean (* execute) (TrackerBatch  *batch,
 			      GCancellable  *cancellable,
 			      GError       **error);
@@ -329,5 +354,7 @@ void tracker_resource_iterator_init (TrackerResourceIterator *iter,
 gboolean tracker_resource_iterator_next (TrackerResourceIterator  *iter,
                                          const gchar             **property,
                                          const GValue            **value);
+const gchar * tracker_resource_get_identifier_internal (TrackerResource *resource);
+gboolean tracker_resource_is_blank_node (TrackerResource *resource);
 
 #endif /* __TRACKER_PRIVATE_H__ */
